<?php

namespace Drupal\poster_integration\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\poster_integration\PosterConnection;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * PosterController
 */
class PosterController extends ControllerBase {

  /**
   * Get list of Products.
   *
   * @return array
   *   render array
   */
  public function getProducts() {
    $connection = new PosterConnection();
    $response = $connection->makeRequest('menu.getProducts', 'get', ['type' => 'products']);
    $rows = [];
    $product_variations = [];
    $header = [
      'id' => $this->t('Product ID'),
      'productid' => $this->t('Commerce product ID'),
      'name' => $this->t('Product name'),
      'product' => $this->t('Commerce product'),
      'price' => $this->t('Price'),
      'category' => $this->t('Category'),
    ];
    $decoded = Json::decode($response);
    $entities = $this->entityTypeManager()
      ->getStorage('commerce_product_variation')
      ->loadMultiple();
    foreach ($entities as $product_variation) {
      $product_variations[$product_variation->get('sku')
        ->getValue()[0]['value']] = [
        $product_variation->get('product_id')
          ->getValue()[0]['target_id'],
        $product_variation->get('title')->getValue()[0]['value'],
        $product_variation->get('sku')->getValue()[0]['value'],
      ];
    }
    foreach ($decoded['response'] as $key => $value) {
      $rows[] = [
        $value['product_id'],
        $product_variations[$value['product_id']][2],
        $value['product_name'],
        $product_variations[$value['product_id']] ? Link::fromTextAndUrl($product_variations[$value['product_id']][1], Url::fromRoute('entity.commerce_product.canonical', ['commerce_product' => $product_variations[$value['product_id']][0]])) : $this->t('No Commerce product'),
        $value['price'][1],
        $value['category_name'],
      ];
    }
    $array_column = array_column($rows, 0);
    array_multisort($array_column, SORT_ASC, $rows);
    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
    ];
  }

}
