<?php

namespace Drupal\poster_integration\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannel;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\poster_integration\PosterConnection;

/**
 * Class OrderCompleteSubscriber.
 *
 * @package Drupal\MY_MODULE
 */
class OrderCompleteSubscriber implements EventSubscriberInterface
{

    /**
     * The Poster configuration.
     *
     * @var \Drupal\Core\Config\ImmutableConfig
     */
    protected $config;

    /**
     * Drupal\Core\Entity\EntityTypeManager definition.
     *
     * @var \Drupal\Core\Entity\EntityTypeManager
     */
    protected $entityTypeManager;

    /**
     * A logger instance.
     *
     * @var \Drupal\Core\Logger\LoggerChannel
     */
    protected $logger;

    /**
     * Constructor.
     */
    public function __construct(EntityTypeManager $entity_type_manager, ConfigFactoryInterface $config_factory, LoggerChannel $logger)
    {
        $this->entityTypeManager = $entity_type_manager;
        $this->config = $config_factory->get('poster_integration.settings');
        $this->logger = $logger;
    }

    /**
     * {@inheritdoc}
     */
    static function getSubscribedEvents()
    {
        $events['commerce_order.place.post_transition'] = ['orderCompleteHandler'];

        return $events;
    }

    /**
     * This method is called whenever the commerce_order.place.post_transition
     * event is dispatched.
     *
     * @param WorkflowTransitionEvent $event
     */
    public function orderCompleteHandler(WorkflowTransitionEvent $event)
    {
        /**
         * @var \Drupal\commerce_order\Entity\OrderInterface $order
         */
        if ($this->config->get('send_orders')) {
            try {
                $order = $event->getEntity();
                // Order items in the cart.
                $items = $order->getItems();
                if($biling_info = $order->getBillingProfile())
                {
                  $customer = $order->getBillingProfile()->get('address');
                } else {
                  throw new \RuntimeException(sprintf('Enable please Collect Billing Information option for "%s" payment gateway', $order->get('payment_gateway')->getValue()[0]['target_id']));
                }
                // Get products from order.
                $products = [];
                if($items)
                {
                  foreach ($items as $key => $value) {
                    $products[] = [
                      'product_id' => $value->getPurchasedEntity()
                        ->get('sku')
                        ->getValue()[0]['value'],
                      'count' => intval($value->get('quantity')->getValue()[0]['value']),
                    ];
                  }
                  $incoming_order = [
                    'spot_id' => $this->config->get('spot_id'),
                    'phone' => $customer->getValue()[0]['family_name'],
                    'first_name' => $customer->getValue()[0]['given_name'],
                    'address' => $customer->getValue()[0]['given_name'] . ' ' . $customer->getValue()[0]['address_line1'] . ' ' . $customer->getValue()[0]['family_name'],
                    'products' => $products,
                  ];
                  $connection = new PosterConnection();
                  $result = $connection->makeRequest('incomingOrders.createIncomingOrder', 'post', $incoming_order);
                  $decoded_result = json_decode($result);
                  if (!empty($decoded_result->error)) {
                    $this->logger->error(
                      'The order @id generates error during Poster API request. @result',
                      [
                        '@id' => $order->id(),
                        '@result' => $result,
                      ]
                    );
                  }
                  else {
                    //@TODO add custom state for orders pushed to Poster
                    //$order->getState()->applyTransitionById('push');
                    //$order->save();
                    $this->logger->debug(
                      'The order @id has been posted to Poster. @result',
                      [
                        '@id' => $order->id(),
                        '@result' => $result,
                      ]
                    );
                  }
                }

            } catch (\Exception $e) {
                $this->logger->error("Poster API error: @message", ['@message' => $e->getMessage()]);
            }

        }
    }

}
