<?php

namespace Drupal\poster_integration;

/**
 * Class PosterConnection
 *
 * @package Drupal\poster_integration
 */
class PosterConnection
{

    const CONNECT_TIMEOUT = 30;
    const ENDPOINT_URL = 'https://joinposter.com/api/';

    /**
     * @var \Drupal\Core\Config\Config Poster settings
     */
    protected $config = null;

    /**
     * The Poster integration logging channel.
     *
     * @var \Drupal\Core\Logger\LoggerChannelInterface
     */
    protected $logger = null;

    /**
     * The HTTP client to fetch the feed data with.
     *
     * @var \GuzzleHttp\ClientInterface
     */
    protected $httpClient;

    /**
     * PosterConnection constructor.
     */
    public function __construct()
    {
        $this->config = \Drupal::config('poster_integration.settings');
        $this->logger = \Drupal::logger('poster_integration');
        $this->httpClient = \Drupal::httpClient();
    }

    /**
     * Make request to Poster API.
     *
     * @param string $method
     * @param string $type
     * @param array  $options
     *
     * @return string
     */
    public function makeRequest($method, $type = 'get', $options = [])
    {

        $url = self::ENDPOINT_URL . $method . '?token=' . $this->config->get('access_token') . '&format=json';
        try {
            if ($type == 'post' || $type == 'put') {
                $optionsres = [
                'headers' => [
                'Content-Type' => 'application/json',
                ],
                'json' => $options
                ];
                $response = $this->httpClient->post($url, $optionsres);
            } else {
                $response = $this->httpClient->get($url, $options);
            }
            if (!$response) {
                throw new \Exception('Empty Response');
            }
        } catch (\Exception $e) {
            $this->logger->error("Couldn't connect to Poster API: @message", ['@message' => $e->getMessage()]);
        }

        return $response ? (string)$response->getBody() : '';
    }

}
